@@ -103,6 +103,11 @@ public:
      */
     ~KateConsole() override;
 
+    /**
+     * sync mode
+     */
+    enum SyncMode { SyncNothing = 0, SyncCurrentTab = 1, SyncCreateTabPerDir = 2 };
+
     void readConfig();
 
     /**
@@ -157,6 +162,21 @@ public Q_SLOTS:
      */
     void slotRun();
 
+    /**
+     * split terminal view vertically
+     */
+    void slotSplitVertical();
+
+    /**
+     * split terminal view horizontally
+     */
+    void slotSplitHorizontal();
+
+    /**
+     * open a new tab
+     */
+    void slotNewTab();
+
 private Q_SLOTS:
     /**
      * the konsole exited ;)
@@ -167,7 +187,7 @@ private Q_SLOTS:
     /**
      * construct console if needed
      */
-    void loadConsoleIfNeeded();
+    void loadConsoleIfNeeded(QString directory = QString());
 
     /**
      * Show or hide the konsole view as appropriate.
@@ -227,6 +247,9 @@ private:
     KateKonsolePlugin *m_plugin;
     QString m_currentPath;
     QMetaObject::Connection m_urlChangedConnection;
+
+    // current sync mode as read from the config
+    SyncMode m_syncMode = SyncNothing;
 };
 
 class KateKonsoleConfigPage : public KTextEditor::ConfigPage
@@ -249,7 +272,7 @@ public:
     }
 
 private:
-    class QCheckBox *cbAutoSyncronize;
+    class QButtonGroup *m_syncMode;
     class QCheckBox *cbRemoveExtension;
     class QLineEdit *lePrefix;
     class QCheckBox *cbSetEditor;
@@ -263,3 +286,5 @@ private Q_SLOTS:
      */
     static void slotEnableRunWarning();
 };
+
+Q_DECLARE_METATYPE(KateConsole::SyncMode)
