@@ -7,6 +7,7 @@
 
 #include "kateprojectindex.h"
 
+#include <QCoreApplication>
 #include <QDir>
 #include <QProcess>
 #include <QStandardPaths>
@@ -31,8 +32,9 @@ KateProjectIndex::KateProjectIndex(const QString &baseDir, const QString &indexD
         }
         m_ctagsIndexFile.reset(new QFile(path));
     } else {
+        auto fileName = QStringLiteral("/kate.project.ctags.%1.%2").arg(QDir(baseDir).dirName(), QCoreApplication::applicationPid());
         // indexDir is typically QDir::tempPath() or otherwise specified in configuration
-        m_ctagsIndexFile.reset(new QTemporaryFile(indexDir + QStringLiteral("/kate.project.ctags")));
+        m_ctagsIndexFile.reset(new QTemporaryFile(indexDir + fileName));
     }
 
     /**
