@@ -86,6 +86,27 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
     });
     hLayout->addWidget(m_tabBar);
 
+    // add Scroll Sync Indicator
+    m_scrollSync = new QToolButton(this);
+    m_scrollSync->setHidden(!m_viewManager->hasScrollSync());
+    m_scrollSync->setCheckable(true);
+    m_scrollSync->setAutoRaise(true);
+    m_scrollSync->setToolTip(i18n("Synchronize Scrolling"));
+    m_scrollSync->setIcon(QIcon::fromTheme(QStringLiteral("link")));
+    m_scrollSync->setWhatsThis(i18n("Synchronize scrolling of this split-view with other synchronized split-views"));
+    connect(m_scrollSync, &QToolButton::toggled, this, [this](bool checked) {
+        if (currentView()) {
+            m_viewManager->slotSynchroniseScrolling(checked, this);
+        } else {
+            // If invalid, undo check
+            const QSignalBlocker blockToggle(m_scrollSync);
+            m_scrollSync->setChecked(!checked);
+        }
+        m_viewManager->updateScrollSyncIndicatorVisibility();
+    });
+    KAcceleratorManager::setNoAccel(m_scrollSync);
+    hLayout->addWidget(m_scrollSync);
+
     // add quick open
     m_quickOpen = new QToolButton(this);
     m_quickOpen->setAutoRaise(true);
@@ -116,13 +137,7 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
     m_split->setToolTip(i18n("Split View"));
     m_split->setWhatsThis(i18n("Control view space splitting"));
 
-    //  add action for Synchronous scrolling. This needs to be one per ViewSpace instance
-    m_toggleSynchronisedScrolling = new KToggleAction(i18n("S&ynchronize Scrolling"), this);
-    m_toggleSynchronisedScrolling->setIcon(QIcon::fromTheme(QStringLiteral("view-sync")));
-    m_toggleSynchronisedScrolling->setWhatsThis(i18n("Synchronize Scrolling of this split-view with other synchronized split-views"));
-    m_split->addAction(m_toggleSynchronisedScrolling);
-    connect(m_toggleSynchronisedScrolling, &QAction::triggered, m_viewManager, &KateViewManager::slotSynchroniseScrolling);
-
+    m_scrollSync->installEventFilter(this); // on click, active this view space
     m_split->installEventFilter(this); // on click, active this view space
     hLayout->addWidget(m_split);
 
@@ -203,37 +218,57 @@ void KateViewSpace::readConfig()
 
 bool KateViewSpace::eventFilter(QObject *obj, QEvent *event)
 {
-    QToolButton *button = qobject_cast<QToolButton *>(obj);
+    if (auto button = qobject_cast<QToolButton *>(obj)) {
+        // quick open button: show tool tip with shortcut
+        if (button == m_quickOpen && event->type() == QEvent::ToolTip) {
+            QHelpEvent *e = static_cast<QHelpEvent *>(event);
+            QAction *quickOpen = m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_quick_open"));
+            Q_ASSERT(quickOpen);
+            QToolTip::showText(e->globalPos(),
+                               button->toolTip() + QStringLiteral(" (%1)").arg(quickOpen->shortcut().toString(QKeySequence::NativeText)),
+                               button);
+            return true;
+        }
 
-    // quick open button: show tool tip with shortcut
-    if (button == m_quickOpen && event->type() == QEvent::ToolTip) {
-        QHelpEvent *e = static_cast<QHelpEvent *>(event);
-        QAction *quickOpen = m_viewManager->mainWindow()->actionCollection()->action(QStringLiteral("view_quick_open"));
-        Q_ASSERT(quickOpen);
-        QToolTip::showText(e->globalPos(), button->toolTip() + QStringLiteral(" (%1)").arg(quickOpen->shortcut().toString(QKeySequence::NativeText)), button);
-        return true;
-    }
+        // quick open button: What's This
+        if (button == m_quickOpen && event->type() == QEvent::WhatsThis) {
+            QHelpEvent *e = static_cast<QHelpEvent *>(event);
+            const int hiddenDocs = hiddenDocuments();
+            QString helpText = (hiddenDocs == 0)
+                ? i18n("Click here to switch to the Quick Open view.")
+                : i18np("Currently, there is one more document open. To see all open documents, switch to the Quick Open view by clicking here.",
+                        "Currently, there are %1 more documents open. To see all open documents, switch to the Quick Open view by clicking here.",
+                        hiddenDocs);
+            QWhatsThis::showText(e->globalPos(), helpText, m_quickOpen);
+            return true;
+        }
 
-    // quick open button: What's This
-    if (button == m_quickOpen && event->type() == QEvent::WhatsThis) {
-        QHelpEvent *e = static_cast<QHelpEvent *>(event);
-        const int hiddenDocs = hiddenDocuments();
-        QString helpText = (hiddenDocs == 0)
-            ? i18n("Click here to switch to the Quick Open view.")
-            : i18np("Currently, there is one more document open. To see all open documents, switch to the Quick Open view by clicking here.",
-                    "Currently, there are %1 more documents open. To see all open documents, switch to the Quick Open view by clicking here.",
-                    hiddenDocs);
-        QWhatsThis::showText(e->globalPos(), helpText, m_quickOpen);
-        return true;
-    }
-
-    // on mouse press on view space bar tool buttons: activate this space
-    if (button && !isActiveSpace() && event->type() == QEvent::MouseButtonPress) {
-        m_viewManager->setActiveSpace(this);
-        if (currentView()) {
-            m_viewManager->activateView(currentView()->document(), this);
+        // on mouse press on view space bar tool buttons: activate this space
+        if (button && !isActiveSpace() && event->type() == QEvent::MouseButtonPress) {
+            m_viewManager->setActiveSpace(this);
+            if (currentView()) {
+                m_viewManager->activateView(currentView()->document(), this);
+            }
         }
     }
+
+    // ensure proper view space activation even for widgets
+    if (event->type() == QEvent::FocusIn && !isActiveSpace() && currentWidget()) {
+        m_viewManager->setActiveSpace(this);
+        activateWidget(currentWidget());
+        return false;
+    }
+
+    // keep track of new sub-widgets
+    if (event->type() == QEvent::ChildAdded || event->type() == QEvent::ChildRemoved) {
+        QChildEvent *c = static_cast<QChildEvent *>(event);
+        if (c->added()) {
+            c->child()->installEventFilter(this);
+        } else if (c->removed()) {
+            c->child()->removeEventFilter(this);
+        }
+    }
+
     return false;
 }
 
@@ -326,12 +361,14 @@ KTextEditor::View *KateViewSpace::createView(KTextEditor::Document *doc)
 
     // restore the config of this view if possible
     if (!m_group.isEmpty()) {
-        QString fn = v->document()->url().toString();
-        if (!fn.isEmpty()) {
-            QString vgroup = QStringLiteral("%1 %2").arg(m_group, fn);
-
-            KateSession::Ptr as = KateApp::self()->sessionManager()->activeSession();
-            if (as->config() && as->config()->hasGroup(vgroup)) {
+        if (KateSession::Ptr as = KateApp::self()->sessionManager()->activeSession(); as->config()) {
+            // try id, fallback to url for old configs
+            QString id = QString::number(KateApp::self()->documentManager()->documentInfo(v->document())->sessionConfigId);
+            QString vgroup = QStringLiteral("%1 %2").arg(m_group, id);
+            if (!as->config()->hasGroup(vgroup)) {
+                vgroup = QStringLiteral("%1 %2").arg(m_group, v->document()->url().toString());
+            }
+            if (as->config()->hasGroup(vgroup)) {
                 KConfigGroup cg(as->config(), vgroup);
                 v->readSessionConfig(cg);
             }
@@ -362,7 +399,7 @@ KTextEditor::View *KateViewSpace::createView(KTextEditor::Document *doc)
     registerDocument(doc);
 
     // view shall still be not registered
-    Q_ASSERT(m_docToView.find(doc) == m_docToView.end());
+    Q_ASSERT(!m_docToView.contains(doc));
 
     // insert View into stack
     stack->addWidget(v);
@@ -619,8 +656,8 @@ bool KateViewSpace::acceptsDroppedTab(const QMimeData *md)
 {
     if (auto tabMimeData = qobject_cast<const TabMimeData *>(md)) {
         return this != tabMimeData->sourceVS && // must not be same viewspace
-            viewManager() == tabMimeData->sourceVS->viewManager() && // for now we don't support dropping into different windows
-            !hasDocument(tabMimeData->doc);
+                                                // for now we don't support dropping into different windows if no document
+            ((viewManager() == tabMimeData->sourceVS->viewManager()) || tabMimeData->doc.doc()) && !hasDocument(tabMimeData->doc);
     }
     return TabMimeData::hasValidData(md);
 }
@@ -647,7 +684,16 @@ void KateViewSpace::dragLeaveEvent(QDragLeaveEvent *e)
 void KateViewSpace::dropEvent(QDropEvent *e)
 {
     if (auto mimeData = qobject_cast<const TabMimeData *>(e->mimeData())) {
-        m_viewManager->moveViewToViewSpace(this, mimeData->sourceVS, mimeData->doc);
+        if (viewManager() == mimeData->sourceVS->viewManager()) {
+            m_viewManager->moveViewToViewSpace(this, mimeData->sourceVS, mimeData->doc);
+        } else {
+            Q_ASSERT(mimeData->doc.doc()); // we ensure that in acceptsDroppedTab
+            auto sourceView = mimeData->sourceVS->findViewForDocument(mimeData->doc.doc());
+            auto view = m_viewManager->activateView(mimeData->doc.doc(), this);
+            if (sourceView && view) {
+                view->setCursorPosition(sourceView->cursorPosition());
+            }
+        }
         m_dropIndicator.reset();
         e->accept();
         return;
@@ -742,7 +788,7 @@ void KateViewSpace::documentDestroyed(QObject *doc)
     /**
      * we shall have no views for this document at this point in time!
      */
-    Q_ASSERT(m_docToView.find(invalidDoc) == m_docToView.end());
+    Q_ASSERT(!m_docToView.contains(invalidDoc));
 
     // disconnect entirely
     disconnect(doc, nullptr, this, nullptr);
@@ -800,6 +846,11 @@ void KateViewSpace::removeWidget(QWidget *w)
     bool shouldClose = true;
     QMetaObject::invokeMethod(w, "shouldClose", Q_RETURN_ARG(bool, shouldClose));
     if (shouldClose) {
+        w->removeEventFilter(this);
+        for (auto c : w->findChildren<QWidget *>()) {
+            c->removeEventFilter(this);
+        }
+
         stack->removeWidget(w);
         m_registeredDocuments.removeOne(w);
 
@@ -863,8 +914,21 @@ void KateViewSpace::focusNextTab()
     }
 }
 
+void KateViewSpace::focusTab(const int id)
+{
+    if (m_tabBar->containsTab(id)) {
+        changeView(id);
+    }
+}
+
 void KateViewSpace::addWidgetAsTab(QWidget *widget)
 {
+    // ensure we keep track of focus changes, KTextEditor::View has some extra focusIn signal
+    widget->installEventFilter(this);
+    for (auto c : widget->findChildren<QWidget *>()) {
+        c->installEventFilter(this);
+    }
+
     stack->addWidget(widget);
     // disconnect changeView, we are just adding the widget here
     // and don't want any unnecessary viewChanged signals
@@ -931,9 +995,14 @@ void KateViewSpace::focusNavigationBar()
     }
 }
 
-void KateViewSpace::setToggleSynchronisedScrollingCheckedState(bool checked)
+void KateViewSpace::toggleScrollSync()
 {
-    m_toggleSynchronisedScrolling->setChecked(checked);
+    m_scrollSync->toggle();
+}
+
+void KateViewSpace::setScrollSyncToolButtonVisible(bool visible)
+{
+    m_scrollSync->setHidden(!visible);
 }
 
 void KateViewSpace::addPositionToHistory(const QUrl &url, KTextEditor::Cursor c, bool calledExternally)
@@ -1155,19 +1224,23 @@ void KateViewSpace::showContextMenu(int idx, const QPoint &globalPos)
 
 void KateViewSpace::saveConfig(KConfigBase *config, int myIndex, const QString &viewConfGrp)
 {
-    //   qCDebug(LOG_KATE)<<"KateViewSpace::saveConfig("<<myIndex<<", "<<viewConfGrp<<") - currentView: "<<currentView()<<")";
-    QString groupname = QString(viewConfGrp + QStringLiteral("-ViewSpace %1")).arg(myIndex);
+    const QString groupname = QString(viewConfGrp + QStringLiteral("-ViewSpace %1")).arg(myIndex);
+    m_group = groupname;
 
-    // aggregate all views in view space (LRU ordered)
+    // aggregate all registered documents & views in view space (LRU ordered)
+    // we need even the documents without tabs to avoid that we later have issues with closing
+    // documents and not the right ones being used as replacement if you have a limit on tabs
     std::vector<KTextEditor::View *> views;
     QStringList lruList;
-    const auto docList = documentList();
-    for (DocOrWidget docOrWidget : docList) {
-        if (docOrWidget.widget()) {
+    const auto docList = m_tabBar->lruSortedDocuments();
+    for (auto doc : docList) {
+        // we can only store stuff about documents that get an id
+        const int sessionId = KateApp::self()->documentManager()->documentInfo(doc)->sessionConfigId;
+        if (sessionId < 0) {
             continue;
         }
-        auto doc = docOrWidget.doc();
-        lruList << doc->url().toString();
+
+        lruList << QString::number(sessionId);
         auto it = m_docToView.find(doc);
         if (it != m_docToView.end()) {
             views.push_back(it->second);
@@ -1179,18 +1252,18 @@ void KateViewSpace::saveConfig(KConfigBase *config, int myIndex, const QString &
     group.writeEntry("Count", static_cast<int>(views.size()));
 
     if (currentView()) {
-        group.writeEntry("Active View", currentView()->document()->url().toString());
+        group.writeEntry("Active View", QString::number(KateApp::self()->documentManager()->documentInfo(currentView()->document())->sessionConfigId));
     }
 
     // Save file list, including cursor position in this instance.
     int idx = 0;
     for (auto view : views) {
-        const auto url = view->document()->url();
-        if (!url.isEmpty()) {
-            group.writeEntry(QStringLiteral("View %1").arg(idx), url.toString());
+        const int sessionId = KateApp::self()->documentManager()->documentInfo(view->document())->sessionConfigId;
+        if (sessionId >= 0) {
+            group.writeEntry(QStringLiteral("View %1").arg(idx), sessionId);
 
-            // view config, group: "ViewSpace <n> url"
-            QString vgroup = QStringLiteral("%1 %2").arg(groupname, url.toString());
+            // view config, group: "ViewSpace <n> id"
+            QString vgroup = QStringLiteral("%1 %2").arg(groupname, QString::number(sessionId));
             KConfigGroup viewGroup(config, vgroup);
             view->writeSessionConfig(viewGroup);
         }
@@ -1214,25 +1287,39 @@ void KateViewSpace::restoreConfig(KateViewManager *viewMan, const KConfigBase *c
 
     // restore Document lru list so that all tabs from the last session reappear
     const QStringList lruList = group.readEntry("Documents", QStringList());
-    for (const auto &url : lruList) {
-        // ignore untitled stuff
-        if (url.isEmpty()) {
+    for (const auto &idOrUrl : lruList) {
+        // ignore stuff with no id or url
+        if (idOrUrl.isEmpty()) {
             continue;
         }
 
-        // ignore non-existing documents
-        if (auto doc = KateApp::self()->documentManager()->findDocument(QUrl(url))) {
+        // if id use that, is the new format to allow to differentiate between different untitled documents
+        bool isInt = false;
+        KTextEditor::Document *doc = nullptr;
+        if (const int id = idOrUrl.toInt(&isInt, 10); isInt) {
+            doc = KateApp::self()->documentManager()->findDocumentForSessionConfigId(id);
+        } else {
+            doc = KateApp::self()->documentManager()->findDocument(QUrl(idOrUrl));
+        }
+        if (doc) {
             registerDocument(doc);
         }
     }
 
     // restore active view properties
-    const QString fn = group.readEntry("Active View");
-    if (!fn.isEmpty()) {
-        if (auto doc = KateApp::self()->documentManager()->findDocument(QUrl(fn))) {
+    if (const QString idOrUrl = group.readEntry("Active View"); !idOrUrl.isEmpty()) {
+        // if id use that, is the new format to allow to differentiate between different untitled documents
+        bool isInt = false;
+        KTextEditor::Document *doc = nullptr;
+        if (const int id = idOrUrl.toInt(&isInt, 10); isInt) {
+            doc = KateApp::self()->documentManager()->findDocumentForSessionConfigId(id);
+        } else {
+            doc = KateApp::self()->documentManager()->findDocument(QUrl(idOrUrl));
+        }
+        if (doc) {
             if (auto view = viewMan->createView(doc, this)) {
                 // view config, group: "ViewSpace <n> url"
-                const QString vgroup = QStringLiteral("%1 %2").arg(groupname, fn);
+                const QString vgroup = QStringLiteral("%1 %2").arg(groupname, idOrUrl);
                 view->readSessionConfig(KConfigGroup(config, vgroup));
                 m_tabBar->setCurrentDocument(doc);
             }
