@@ -16,6 +16,7 @@
 #include <QTimer>
 
 #include <KSyntaxHighlighting/Definition>
+#include <KSyntaxHighlighting/Format>
 #include <KSyntaxHighlighting/Repository>
 #include <KSyntaxHighlighting/SyntaxHighlighter>
 #include <KTextEditor/Document>
@@ -25,6 +26,29 @@
 #include <QMenu>
 #include <QScopedValueRollback>
 
+class TooltipHighlighter final : public KSyntaxHighlighting::SyntaxHighlighter
+{
+public:
+    using KSyntaxHighlighting::SyntaxHighlighter::SyntaxHighlighter;
+
+    void highlightBlock(const QString &text) override
+    {
+        const auto fmt = currentBlock().blockFormat();
+        if (fmt.property(QTextFormat::BlockCodeLanguage).isValid()) {
+            // highlight blocks marked with BlockCodeLanguage format
+            return KSyntaxHighlighting::SyntaxHighlighter::highlightBlock(text);
+        }
+    }
+
+    void applyFormat(int offset, int length, const KSyntaxHighlighting::Format &format) override
+    {
+        if (format.textStyle() == KSyntaxHighlighting::Theme::TextStyle::Error) {
+            return;
+        }
+        KSyntaxHighlighting::SyntaxHighlighter::applyFormat(offset, length, format);
+    }
+};
+
 class TooltipPrivate : public QTextBrowser
 {
 public:
@@ -39,7 +63,6 @@ public:
             setPlainText(text);
         } else {
             QString htext = text;
-            htext.replace(QLatin1Char('\n'), QStringLiteral("  \n"));
             setMarkdown(htext);
         }
         resizeTip(text);
@@ -87,7 +110,7 @@ public:
 
             m_view = view;
 
-            hl.setDefinition(KTextEditor::Editor::instance()->repository().definitionForFileName(m_view->document()->url().toString()));
+            hl->setDefinition(KTextEditor::Editor::instance()->repository().definitionForFileName(m_view->document()->url().toString()));
 
             if (m_view && m_view->focusProxy()) {
                 m_view->focusProxy()->installEventFilter(this);
@@ -97,7 +120,7 @@ public:
 
     TooltipPrivate(QWidget *parent, bool manual)
         : QTextBrowser(parent)
-        , hl(document())
+        , hl(new TooltipHighlighter(document()))
         , m_manual(manual)
     {
         setWindowFlags(Qt::FramelessWindowHint | Qt::BypassGraphicsProxyWidget | Qt::ToolTip);
@@ -114,7 +137,7 @@ public:
 
         auto updateColors = [this](KTextEditor::Editor *e) {
             auto theme = e->theme();
-            hl.setTheme(theme);
+            hl->setTheme(theme);
 
             auto pal = palette();
             const QColor bg = theme.editorColor(KSyntaxHighlighting::Theme::BackgroundColor);
@@ -129,7 +152,7 @@ public:
         connect(KTextEditor::Editor::instance(), &KTextEditor::Editor::configChanged, this, updateColors);
     }
 
-    bool eventFilter(QObject *, QEvent *e) override
+    bool eventFilter(QObject *o, QEvent *e) override
     {
         switch (e->type()) {
         // only consider KeyPress
@@ -146,19 +169,23 @@ public:
             }
             break;
         case QEvent::MouseMove: {
-            auto me = static_cast<QMouseEvent *>(e);
-            if (!m_manual && !hasFocus() && !rect().contains(me->position().toPoint())) {
+            if (o == verticalScrollBar() || o == horizontalScrollBar()) {
+                return false;
+            }
+            auto pos = mapFromGlobal(static_cast<QSinglePointEvent *>(e)->globalPosition()).toPoint();
+            if (!m_manual && !hasFocus() && !rect().contains(pos)) {
                 hideTooltipWithDelay();
             }
         } break;
         case QEvent::MouseButtonPress:
         case QEvent::MouseButtonRelease:
         case QEvent::MouseButtonDblClick:
-        case QEvent::Wheel:
-            if (!rect().contains(static_cast<QSinglePointEvent *>(e)->position().toPoint())) {
+        case QEvent::Wheel: {
+            auto pos = mapFromGlobal(static_cast<QSinglePointEvent *>(e)->globalPosition()).toPoint();
+            if (!rect().contains(pos)) {
                 hideTooltip();
             }
-            break;
+        } break;
         default:
             break;
         }
@@ -272,7 +299,7 @@ private:
     bool inContextMenu = false;
     QPointer<KTextEditor::View> m_view;
     QTimer m_hideTimer;
-    KSyntaxHighlighting::SyntaxHighlighter hl;
+    KSyntaxHighlighting::SyntaxHighlighter *hl;
     bool m_manual;
     TextHintMarkupKind m_kind = TextHintMarkupKind::PlainText;
 };
